{% extends "base.html" %}

{% block content %}

  <h1 id="404-page-not-found" style="text-align: center">404</h1>
  <p style="text-align: center"><strong>Page not found</strong></p>
  <p style="text-align: center">Try the <a href="http://www.django-rest-framework.org/">homepage</a>, or <a href="#searchModal" data-toggle="modal">search the documentation</a>.</p>

{% endblock %}
